@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2024  Selwin van Dijk
+  Copyright (C) 2024-2025  Selwin van Dijk
 
   This file is part of signalbackup-tools.
 
@@ -42,6 +42,7 @@ Arg::Arg(int argc, char *argv[])
   d_mergegroups(std::vector<std::string>()),
   d_exportcsv(std::vector<std::pair<std::string,std::string>>()),
   d_exportxml(std::string()),
+  d_querymode(std::string()),
   d_runsqlquery(std::vector<std::string>()),
   d_runprettysqlquery(std::vector<std::string>()),
   d_rundtsqlquery(std::vector<std::string>()),
@@ -53,6 +54,7 @@ Arg::Arg(int argc, char *argv[])
   d_desktopdir(std::string()),
   d_desktopdirs_1(std::string()),
   d_desktopdirs_2(std::string()),
+  d_rawdesktopdb(std::string()),
   d_desktopkey(std::string()),
   d_showdesktopkey(false),
   d_dumpmedia(std::string()),
@@ -77,6 +79,8 @@ Arg::Arg(int argc, char *argv[])
   d_strugee3(-1),
   d_ashmorgan(false),
   d_strugee2(false),
+  d_hiperfall(-1),
+  d_arc(-1),
   d_deleteattachments(false),
   d_onlyinthreads(std::vector<long long int>()),
   d_onlyolderthan(std::string()),
@@ -100,9 +104,8 @@ Arg::Arg(int argc, char *argv[])
   d_interactive(false),
   d_exporthtml(std::string()),
   d_exportdesktophtml(std::string()),
-  d_exportplaintextbackuphtml_1(std::string()),
-  d_exportplaintextbackuphtml_2(std::string()),
-  d_importplaintextbackup(std::string()),
+  d_exportplaintextbackuphtml(std::vector<std::string>()),
+  d_importplaintextbackup(std::vector<std::string>()),
   d_addexportdetails(false),
   d_includecalllog(false),
   d_includeblockedlist(false),
@@ -144,9 +147,18 @@ Arg::Arg(int argc, char *argv[])
   d_skipmessagereorder(false),
   d_migrate_to_191(false),
   d_mapxmlcontacts(std::vector<std::pair<std::string,long long int>>()),
-  d_listxmlcontacts(std::string()),
+  d_listxmlcontacts(std::vector<std::string>()),
   d_selectxmlchats(std::vector<std::string>()),
   d_linkify(true),
+  d_setchatcolors(std::vector<std::pair<long long int, std::string>>()),
+  d_mapxmlcontactnames(std::vector<std::pair<std::string, std::string>>()),
+  d_mapxmlcontactnamesfromfile(std::string()),
+  d_xmlautogroupnames(false),
+  d_setcountrycode(std::string()),
+  d_compactfilenames(false),
+  d_generatedummy(std::string()),
+  d_targetisdummy(false),
+  d_htmlignoremediatypes(std::vector<std::string>()),
   d_input_required(false)
 {
   // vector to hold arguments
@@ -163,13 +175,14 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
 
   bool ok = true;
 
-  for (size_t i = 0; i < arguments.size(); ++i)
+  int argsize = arguments.size();
+  for (int i = 0; i < argsize; ++i)
   {
     std::string option = arguments[i];
 
     if (option == "-i" || option == "--input")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         d_input = std::move(arguments[++i]);
       }
@@ -182,7 +195,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "-p" || option == "--passphrase" || option == "--password")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         d_passphrase = std::move(arguments[++i]);
       }
@@ -195,7 +208,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--importthreads")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         if (arguments[i + 1] == "all" || arguments[i + 1] == "ALL")
         {
@@ -227,7 +240,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--importthreadsbyname")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         if (!parseStringList(arguments[++i], &d_importthreadsbyname))
         {
@@ -245,7 +258,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--limittothreads")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         if (!parseNumberList(arguments[++i], &d_limittothreads, true))
         {
@@ -262,7 +275,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--limittothreadsbyname")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         if (!parseStringList(arguments[++i], &d_limittothreadsbyname))
         {
@@ -279,7 +292,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "-o" || option == "--output")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         d_output = std::move(arguments[++i]);
       }
@@ -293,7 +306,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "-op" || option == "--opassphrase" || option == "--opassword")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         d_opassphrase = std::move(arguments[++i]);
       }
@@ -306,7 +319,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "-s" || option == "--source")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         d_source = std::move(arguments[++i]);
       }
@@ -319,7 +332,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "-sp" || option == "--sourcepassphrase" || option == "--sourcepassword")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         d_sourcepassphrase = std::move(arguments[++i]);
       }
@@ -332,7 +345,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--croptothreads")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         if (!parseNumberList(arguments[++i], &d_croptothreads, true))
         {
@@ -350,7 +363,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--croptothreadsbyname")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         if (!parseStringList(arguments[++i], &d_croptothreadsbyname))
         {
@@ -368,7 +381,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--croptodates")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         std::regex validator("^(?:(?:[0-9]{4}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2})|[0-9]+), *(?:(?:[0-9]{4}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2})|[0-9]+)(?:, *(?:(?:[0-9]{4}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2})|[0-9]+), *(?:(?:[0-9]{4}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2})|[0-9]+))*$");
         if (!std::regex_match(arguments[i + 1], validator))
@@ -393,7 +406,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--mergerecipients")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         if (!parseStringList(arguments[++i], &d_mergerecipients))
         {
@@ -411,7 +424,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--mergegroups")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         if (!parseStringList(arguments[++i], &d_mergegroups))
         {
@@ -429,7 +442,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--exportcsv")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         std::string error;
         if (!parsePairList(arguments[++i], "=", &d_exportcsv, &error))
@@ -448,7 +461,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--exportxml")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         d_exportxml = std::move(arguments[++i]);
       }
@@ -460,11 +473,31 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
       d_input_required = true;
       continue;
     }
+    if (option == "--querymode")
+    {
+      if (i < argsize - 1)
+      {
+        std::regex validator("line|pretty|single", std::regex::icase);
+        if (!std::regex_match(arguments[i + 1], validator))
+        {
+          std::cerr << "[ Error parsing command line option `" << option << "': Bad argument. ]" << std::endl;
+          ok = false;
+          continue;
+        }
+        d_querymode = std::move(arguments[++i]);
+      }
+      else
+      {
+        std::cerr << "[ Error parsing command line option `" << option << "': Missing argument. ]" << std::endl;
+        ok = false;
+      }
+      continue;
+    }
     if (option == "--runsqlquery")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
-          d_runsqlquery.emplace_back(std::move(arguments[++i]));
+        d_runsqlquery.emplace_back(std::move(arguments[++i]));
       }
       else
       {
@@ -476,9 +509,9 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--runprettysqlquery")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
-          d_runprettysqlquery.emplace_back(std::move(arguments[++i]));
+        d_runprettysqlquery.emplace_back(std::move(arguments[++i]));
       }
       else
       {
@@ -490,9 +523,9 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--rundtsqlquery")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
-          d_rundtsqlquery.emplace_back(std::move(arguments[++i]));
+        d_rundtsqlquery.emplace_back(std::move(arguments[++i]));
       }
       else
       {
@@ -503,9 +536,9 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--rundtprettysqlquery")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
-          d_rundtprettysqlquery.emplace_back(std::move(arguments[++i]));
+        d_rundtprettysqlquery.emplace_back(std::move(arguments[++i]));
       }
       else
       {
@@ -516,7 +549,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--limitcontacts")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         if (!parseStringList(arguments[++i], &d_limitcontacts))
         {
@@ -543,7 +576,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--editattachmentsize")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         if (!parseNumberList(arguments[++i], &d_editattachmentsize, false))
         {
@@ -560,7 +593,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--dumpdesktopdb")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         d_dumpdesktopdb = std::move(arguments[++i]);
       }
@@ -573,7 +606,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--desktopdir")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         d_desktopdir = std::move(arguments[++i]);
         d_desktopdirs_1 = d_desktopdir;
@@ -588,7 +621,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--desktopdirs")
     {
-      if (i < arguments.size() - 2)
+      if (i < argsize - 2)
       {
         d_desktopdirs_1 = std::move(arguments[++i]);
         d_desktopdirs_2 = std::move(arguments[++i]);
@@ -600,9 +633,22 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
       }
       continue;
     }
+    if (option == "--rawdesktopdb")
+    {
+      if (i < argsize - 1)
+      {
+        d_rawdesktopdb = std::move(arguments[++i]);
+      }
+      else
+      {
+        std::cerr << "[ Error parsing command line option `" << option << "': Missing argument. ]" << std::endl;
+        ok = false;
+      }
+      continue;
+    }
     if (option == "--desktopkey")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         std::regex validator("^[0-9a-fA-F]{64}$", std::regex::icase);
         if (!std::regex_match(arguments[i + 1], validator))
@@ -632,7 +678,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--dumpmedia")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         d_dumpmedia = std::move(arguments[++i]);
       }
@@ -656,7 +702,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--dumpavatars")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         d_dumpavatars = std::move(arguments[++i]);
       }
@@ -680,7 +726,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--importcsv")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         d_importcsv = std::move(arguments[++i]);
       }
@@ -694,7 +740,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--mapcsvfields")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         std::string error;
         if (!parsePairList(arguments[++i], "=", &d_mapcsvfields, &error))
@@ -712,7 +758,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--setselfid")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         d_setselfid = std::move(arguments[++i]);
       }
@@ -778,7 +824,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     if (option == "--removedoubles")
     {
       d_removedoubles_bool = true;
-      if (i < arguments.size() - 1 && !isOption(arguments[i + 1]))
+      if (i < argsize - 1 && !isOption(arguments[i + 1]))
       {
         if (!ston(&d_removedoubles, arguments[++i]))
         {
@@ -832,7 +878,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--strugee")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         if (!ston(&d_strugee, arguments[++i]))
         {
@@ -849,7 +895,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--strugee3")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         if (!ston(&d_strugee3, arguments[++i]))
         {
@@ -884,6 +930,40 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
       d_strugee2 = false;
       continue;
     }
+    if (option == "--hiperfall")
+    {
+      if (i < argsize - 1)
+      {
+        if (!ston(&d_hiperfall, arguments[++i]))
+        {
+          std::cerr << "[ Error parsing command line option `" << option << "': Bad argument. ]" << std::endl;
+          ok = false;
+        }
+      }
+      else
+      {
+        std::cerr << "[ Error parsing command line option `" << option << "': Missing argument. ]" << std::endl;
+        ok = false;
+      }
+      continue;
+    }
+    if (option == "--arc")
+    {
+      if (i < argsize - 1)
+      {
+        if (!ston(&d_arc, arguments[++i]))
+        {
+          std::cerr << "[ Error parsing command line option `" << option << "': Bad argument. ]" << std::endl;
+          ok = false;
+        }
+      }
+      else
+      {
+        std::cerr << "[ Error parsing command line option `" << option << "': Missing argument. ]" << std::endl;
+        ok = false;
+      }
+      continue;
+    }
     if (option == "--deleteattachments")
     {
       d_deleteattachments = true;
@@ -897,7 +977,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--onlyinthreads")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         if (!parseNumberList(arguments[++i], &d_onlyinthreads, true))
         {
@@ -914,7 +994,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--onlyolderthan")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         std::regex validator("^(?:(?:[0-9]{4}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2})|[0-9]+)$", std::regex::icase);
         if (!std::regex_match(arguments[i + 1], validator))
@@ -934,7 +1014,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--onlynewerthan")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         std::regex validator("^(?:(?:[0-9]{4}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2})|[0-9]+)$", std::regex::icase);
         if (!std::regex_match(arguments[i + 1], validator))
@@ -954,7 +1034,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--onlylargerthan")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         if (!ston(&d_onlylargerthan, arguments[++i]))
         {
@@ -971,7 +1051,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--onlytype")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         if (!parseStringList(arguments[++i], &d_onlytype))
         {
@@ -988,7 +1068,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--appendbody")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         d_appendbody = std::move(arguments[++i]);
       }
@@ -1001,7 +1081,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--prependbody")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         d_prependbody = std::move(arguments[++i]);
       }
@@ -1014,7 +1094,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--replaceattachments")
     {
-      if (i < arguments.size() - 1 && !isOption(arguments[i + 1]))
+      if (i < argsize - 1 && !isOption(arguments[i + 1]))
       {
         std::string error;
         if (!parsePairList(arguments[++i], "=", &d_replaceattachments, &error))
@@ -1085,7 +1165,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--limittodates")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         std::regex validator("^(?:(?:[0-9]{4}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2})|[0-9]+), *(?:(?:[0-9]{4}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2})|[0-9]+)(?:, *(?:(?:[0-9]{4}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2})|[0-9]+), *(?:(?:[0-9]{4}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2})|[0-9]+))*$");
         if (!std::regex_match(arguments[i + 1], validator))
@@ -1160,7 +1240,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--exporthtml")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         d_exporthtml = std::move(arguments[++i]);
       }
@@ -1174,7 +1254,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--exportdesktophtml")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         d_exportdesktophtml = std::move(arguments[++i]);
       }
@@ -1187,27 +1267,26 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--exportplaintextbackuphtml")
     {
-      if (i < arguments.size() - 2)
+      while (i < argsize - 1 && !isOption(arguments[i + 1]))
       {
-        d_exportplaintextbackuphtml_1 = std::move(arguments[++i]);
-        d_exportplaintextbackuphtml_2 = std::move(arguments[++i]);
+        d_exportplaintextbackuphtml.emplace_back(std::move(arguments[++i]));
       }
-      else
+      if (d_exportplaintextbackuphtml.size() < 2)
       {
-        std::cerr << "[ Error parsing command line option `" << option << "': Missing argument. ]" << std::endl;
+        std::cerr << "[ Error parsing command line option `" << option << "': 2 arguments required, " << d_exportplaintextbackuphtml.size() << " provided ]" << std::endl;
         ok = false;
       }
       continue;
     }
     if (option == "--importplaintextbackup")
     {
-      if (i < arguments.size() - 1)
+      while (i < argsize - 1 && !isOption(arguments[i + 1]))
       {
-        d_importplaintextbackup = std::move(arguments[++i]);
+        d_importplaintextbackup.emplace_back(std::move(arguments[++i]));
       }
-      else
+      if (d_importplaintextbackup.size() < 1)
       {
-        std::cerr << "[ Error parsing command line option `" << option << "': Missing argument. ]" << std::endl;
+        std::cerr << "[ Error parsing command line option `" << option << "': 1 arguments required, " << d_importplaintextbackup.size() << " provided ]" << std::endl;
         ok = false;
       }
       d_input_required = true;
@@ -1328,7 +1407,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     if (option == "--split")
     {
       d_split_bool = true;
-      if (i < arguments.size() - 1 && !isOption(arguments[i + 1]))
+      if (i < argsize - 1 && !isOption(arguments[i + 1]))
       {
         if (!ston(&d_split, arguments[++i]))
         {
@@ -1341,7 +1420,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--split-by")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         std::regex validator("year|month|week|day", std::regex::icase);
         if (!std::regex_match(arguments[i + 1], validator))
@@ -1403,7 +1482,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--exporttxt")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         d_exporttxt = std::move(arguments[++i]);
       }
@@ -1417,7 +1496,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--exportdesktoptxt")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         d_exportdesktoptxt = std::move(arguments[++i]);
       }
@@ -1440,7 +1519,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--desktopdbversion")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         if (!ston(&d_desktopdbversion, arguments[++i]))
         {
@@ -1477,7 +1556,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--findrecipient")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         if (!ston(&d_findrecipient, arguments[++i]))
         {
@@ -1494,7 +1573,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--importtelegram" || option == "--importjson")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         d_importtelegram = std::move(arguments[++i]);
       }
@@ -1508,7 +1587,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--listjsonchats")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         d_listjsonchats = std::move(arguments[++i]);
       }
@@ -1521,7 +1600,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--selectjsonchats")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         if (!parseNumberList(arguments[++i], &d_selectjsonchats, true))
         {
@@ -1538,7 +1617,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--mapjsoncontacts")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         std::string error;
         if (!parsePairList(arguments[++i], "=", &d_mapjsoncontacts, &error))
@@ -1556,7 +1635,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--preventjsonmapping")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         if (!parseStringList(arguments[++i], &d_preventjsonmapping))
         {
@@ -1634,7 +1713,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "-l" || option == "--logfile")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         d_logfile = std::move(arguments[++i]);
       }
@@ -1689,7 +1768,7 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--mapxmlcontacts")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         std::string error;
         if (!parsePairList(arguments[++i], "=", &d_mapxmlcontacts, &error))
@@ -1707,20 +1786,20 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
     }
     if (option == "--listxmlcontacts")
     {
-      if (i < arguments.size() - 1)
+      while (i < argsize - 1 && !isOption(arguments[i + 1]))
       {
-        d_listxmlcontacts = std::move(arguments[++i]);
+        d_listxmlcontacts.emplace_back(std::move(arguments[++i]));
       }
-      else
+      if (d_listxmlcontacts.size() < 1)
       {
-        std::cerr << "[ Error parsing command line option `" << option << "': Missing argument. ]" << std::endl;
+        std::cerr << "[ Error parsing command line option `" << option << "': 1 arguments required, " << d_listxmlcontacts.size() << " provided ]" << std::endl;
         ok = false;
       }
       continue;
     }
     if (option == "--selectxmlchats")
     {
-      if (i < arguments.size() - 1)
+      if (i < argsize - 1)
       {
         if (!parseStringList(arguments[++i], &d_selectxmlchats))
         {
@@ -1745,6 +1824,131 @@ bool Arg::parseArgs(std::vector<std::string> const &arguments)
       d_linkify = false;
       continue;
     }
+    if (option == "--setchatcolors")
+    {
+      if (i < argsize - 1)
+      {
+        std::string error;
+        if (!parsePairList(arguments[++i], "=", &d_setchatcolors, &error))
+        {
+          std::cerr << "[ Error parsing command line option `" << option << "': " << error << " ]" << std::endl;
+          ok = false;
+        }
+      }
+      else
+      {
+        std::cerr << "[ Error parsing command line option `" << option << "': Missing argument. ]" << std::endl;
+        ok = false;
+      }
+      continue;
+    }
+    if (option == "--mapxmlcontactnames")
+    {
+      if (i < argsize - 1)
+      {
+        std::string error;
+        if (!parsePairList(arguments[++i], "=", &d_mapxmlcontactnames, &error))
+        {
+          std::cerr << "[ Error parsing command line option `" << option << "': " << error << " ]" << std::endl;
+          ok = false;
+        }
+      }
+      else
+      {
+        std::cerr << "[ Error parsing command line option `" << option << "': Missing argument. ]" << std::endl;
+        ok = false;
+      }
+      continue;
+    }
+    if (option == "--mapxmlcontactnamesfromfile")
+    {
+      if (i < argsize - 1)
+      {
+        d_mapxmlcontactnamesfromfile = std::move(arguments[++i]);
+      }
+      else
+      {
+        std::cerr << "[ Error parsing command line option `" << option << "': Missing argument. ]" << std::endl;
+        ok = false;
+      }
+      continue;
+    }
+    if (option == "--xmlautogroupnames")
+    {
+      d_xmlautogroupnames = true;
+      continue;
+    }
+    if (option == "--no-xmlautogroupnames")
+    {
+      d_xmlautogroupnames = false;
+      continue;
+    }
+    if (option == "--setcountrycode")
+    {
+      if (i < argsize - 1)
+      {
+        d_setcountrycode = std::move(arguments[++i]);
+      }
+      else
+      {
+        std::cerr << "[ Error parsing command line option `" << option << "': Missing argument. ]" << std::endl;
+        ok = false;
+      }
+      continue;
+    }
+    if (option == "--compactfilenames")
+    {
+      d_compactfilenames = true;
+      continue;
+    }
+    if (option == "--no-compactfilenames")
+    {
+      d_compactfilenames = false;
+      continue;
+    }
+    if (option == "--generatedummy")
+    {
+      if (i < argsize - 1)
+      {
+        d_generatedummy = std::move(arguments[++i]);
+        d_opassphrase = "000000000000000000000000000001";
+      }
+      else
+      {
+        std::cerr << "[ Error parsing command line option `" << option << "': Missing argument. ]" << std::endl;
+        ok = false;
+      }
+      continue;
+    }
+    if (option == "--targetisdummy")
+    {
+      d_targetisdummy = true;
+      d_passphrase = "000000000000000000000000000001";
+      d_opassphrase = "000000000000000000000000000001";
+      continue;
+    }
+    if (option == "--no-targetisdummy")
+    {
+      d_targetisdummy = false;
+      continue;
+    }
+    if (option == "--htmlignoremediatypes")
+    {
+      if (i < argsize - 1)
+      {
+        if (!parseStringList(arguments[++i], &d_htmlignoremediatypes))
+        {
+          std::cerr << "[ Error parsing command line option `" << option << "': Bad argument. ]" << std::endl;
+          ok = false;
+        }
+      }
+      else
+      {
+        std::cerr << "[ Error parsing command line option `" << option << "': Missing argument. ]" << std::endl;
+        ok = false;
+      }
+      continue;
+    }
     if (option[0] != '-')
     {
       if (d_positionals >= 2)
