@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2019-2024  Selwin van Dijk
+  Copyright (C) 2019-2025  Selwin van Dijk
 
   This file is part of signalbackup-tools.
 
@@ -20,6 +20,10 @@
 #include <string>
 #include <vector>
 
+#if __cpp_lib_span >= 202002L
+#include <span>
+#endif
+
 #include "main.h"
 #include "arg/arg.h"
 #include "common_be.h"
@@ -55,9 +59,9 @@ int main(int argc, char *argv[])
 
 #ifdef VERSIONDATE
   #if defined(_WIN32) || defined(__MINGW64__)
-  Logger::message("signalbackup-tools (", argv[0], ") source version ", VERSIONDATE, " (Win)", " (SQlite: ", SQLITE_VERSION, ", OpenSSL: ", OPENSSL_VERSION_TEXT, ")");
+  Logger::message("signalbackup-tools (", argv[0], ") source version ", VERSIONDATE, " (Win)", " (SQLite: ", SQLITE_VERSION, ", OpenSSL: ", OPENSSL_VERSION_TEXT, ")");
   #else
-  Logger::message("signalbackup-tools (", argv[0], ") source version ", VERSIONDATE, " (SQlite: ", SQLITE_VERSION, ", OpenSSL: ", OPENSSL_VERSION_TEXT, ")");
+  Logger::message("signalbackup-tools (", argv[0], ") source version ", VERSIONDATE, " (SQLite: ", SQLITE_VERSION, ", OpenSSL: ", OPENSSL_VERSION_TEXT, ")");
   #endif
 #endif
 
@@ -81,24 +85,37 @@ int main(int argc, char *argv[])
     return 0;
   }
 
-  //**** OPTIONS THAT DO NOT REQUIRE SIGNAL BACKUP AS INPUT ****//
+  //**** OPTIONS THAT DO NOT REQUIRE SIGNAL BACKUP  INPUT ****//
   std::unique_ptr<DesktopDatabase> ddb;
   std::unique_ptr<SignalPlaintextBackupDatabase> ptdb;
   auto initDesktopDatabase = [&]()
   {
     if (!ddb)
-      ddb.reset(new DesktopDatabase(arg.desktopdirs_1(), arg.desktopdirs_2(), arg.desktopkey(), arg.verbose(),
-                                    arg.ignorewal(), arg.desktopdbversion(), arg.truncate(), arg.showdesktopkey(),
-                                    arg.dbusverbose()));
+      ddb.reset(new DesktopDatabase(arg.desktopdirs_1(), arg.desktopdirs_2(), arg.rawdesktopdb(), arg.desktopkey(),
+                                    arg.verbose(), arg.ignorewal(), arg.desktopdbversion(), arg.truncate(),
+                                    arg.showdesktopkey(), arg.dbusverbose()));
     return ddb->ok();
   };
-  auto initPlaintextDatabase = [&](std::string const &xmlfile)
+#if __cpp_lib_span >= 202002L
+  auto initPlaintextDatabase = [&](std::span<std::string const> const &xmlfiles)
+#else
+  auto initPlaintextDatabase = [&](std::vector<std::string> const &xmlfiles)
+#endif
   {
     if (!ptdb)
-      ptdb.reset(new SignalPlaintextBackupDatabase(xmlfile, arg.truncate(), arg.verbose()));
+      ptdb.reset(new SignalPlaintextBackupDatabase(xmlfiles, arg.truncate(), arg.verbose(), arg.mapxmlcontactnames(),
+                                                   arg.mapxmlcontactnamesfromfile(), arg.setcountrycode(), arg.xmlautogroupnames()));
     return ptdb->ok();
   };
 
+  if (!arg.generatedummy().empty())
+  {
+    DummyBackup d(arg.verbose(), arg.truncate(), arg.showprogress());
+    if (!d.ok() ||
+        !d.exportBackup(arg.generatedummy(), arg.opassphrase(), arg.overwrite(), SignalBackup::DROPATTACHMENTDATA, false /*onlydb*/))
+      return 1;
+  }
+
   // show desktop key
   if (arg.showdesktopkey())
     if (!initDesktopDatabase())
@@ -111,7 +128,7 @@ int main(int argc, char *argv[])
       return 1;
 
     for (auto const &q : arg.rundtsqlquery())
-      ddb->runQuery(q, false);
+      ddb->runQuery(q, arg.querymode());
   }
   if (!arg.rundtprettysqlquery().empty())
   {
@@ -119,7 +136,7 @@ int main(int argc, char *argv[])
       return 1;
 
     for (auto const &q : arg.rundtprettysqlquery())
-      ddb->runQuery(q, true);
+      ddb->runQuery(q, "pretty");
   }
 
   if (!arg.dumpdesktopdb().empty())
@@ -153,10 +170,15 @@ int main(int argc, char *argv[])
 
     DummyBackup dummydb(ddb, arg.verbose(), arg.truncate(), arg.showprogress());
     if (!dummydb.ok())
+    {
+      if (arg.verbose()) [[unlikely]]
+        Logger::error("DummyBackup not initialized ok");
       return 1;
+    }
 
     if (!dummydb.importFromDesktop(ddb, true /*arg.skipmessagereorder()*/, arg.limittodates(), true /*addincompletedata*/,
-                                   true /*importcontacts*/, false /*autolimittodates*/, true /*importstickers*/, arg.setselfid()))
+                                   false /*importcontacts*/, false /*autolimittodates*/, true /*importstickers*/, arg.setselfid(),
+                                   true /*targetisdummy*/))
       return 1;
 
     if (!arg.exportdesktophtml().empty())
@@ -164,7 +186,8 @@ int main(int argc, char *argv[])
                               (arg.split_bool() ? arg.split() : -1), arg.setselfid(),  arg.includecalllog(), arg.searchpage(),
                               arg.stickerpacks(), arg.migratedb(), arg.overwrite(), arg.append(), arg.light(), arg.themeswitching(),
                               arg.addexportdetails(), arg.includeblockedlist(), arg.includefullcontactlist(), false /*arg.includesettings()*/,
-                              arg.includereceipts(), arg.originalfilenames(), arg.linkify(), arg.chatfolders()))
+                              arg.includereceipts(), arg.originalfilenames(), arg.linkify(), arg.chatfolders(), arg.compactfilenames(),
+                              arg.htmlignoremediatypes()))
         return 1;
 
     if (!arg.exportdesktoptxt().empty())
@@ -173,25 +196,32 @@ int main(int argc, char *argv[])
         return 1;
   }
 
-  if (!arg.exportplaintextbackuphtml_1().empty())
+  if (!arg.exportplaintextbackuphtml().empty())
   {
-    if (!initPlaintextDatabase(arg.exportplaintextbackuphtml_1()))
+    // skip last entry, it is the output
+#if __cpp_lib_span >= 202002L
+    if (!initPlaintextDatabase(std::span(arg.exportplaintextbackuphtml().begin(), arg.exportplaintextbackuphtml().end() - 1)))
+#else
+    std::vector<std::string> xmlfiles(arg.exportplaintextbackuphtml().begin(), arg.exportplaintextbackuphtml().end() - 1);
+    if (!initPlaintextDatabase(xmlfiles))
+#endif
       return 1;
 
-    DummyBackup dummydb(arg.setselfid(), arg.verbose(), arg.truncate(), arg.showprogress());
+    DummyBackup dummydb(ptdb, arg.setselfid(), arg.verbose(), arg.truncate(), arg.showprogress());
     if (!dummydb.ok())
       return 1;
 
     if (!dummydb.importFromPlaintextBackup(ptdb, true /*arg.skipmessagereorder()*/, arg.mapxmlcontacts(), arg.limittodates(),
                                            arg.selectxmlchats(), true /*addincompletedata*/, arg.xmlmarkdelivered(),
-                                           arg.xmlmarkread(), false /*autolimittodates*/, arg.setselfid()))
+                                           arg.xmlmarkread(), false /*autolimittodates*/, arg.setselfid(), true /*isdummydb*/))
       return 1;
 
-    if (!dummydb.exportHtml(arg.exportplaintextbackuphtml_2(), {} /*limittothreads*/, arg.limittodates(), arg.split_by(),
+    if (!dummydb.exportHtml(arg.exportplaintextbackuphtml().back(), {} /*limittothreads*/, arg.limittodates(), arg.split_by(),
                             (arg.split_bool() ? arg.split() : -1), arg.setselfid(), arg.includecalllog(), arg.searchpage(),
                             arg.stickerpacks(), arg.migratedb(), arg.overwrite(), arg.append(), arg.light(), arg.themeswitching(),
                             arg.addexportdetails(), arg.includeblockedlist(), arg.includefullcontactlist(), false /*arg.includesettings()*/,
-                            arg.includereceipts(), arg.originalfilenames(), arg.linkify(), arg.chatfolders()))
+                            arg.includereceipts(), arg.originalfilenames(), arg.linkify(), arg.chatfolders(), arg.compactfilenames(),
+                            arg.htmlignoremediatypes()))
       return 1;
   }
 
@@ -393,7 +423,8 @@ int main(int argc, char *argv[])
     MEMINFO("Before importfromdesktop");
     if (!sb->importFromDesktop(ddb, arg.skipmessagereorder(), arg.limittodates(),
                                (arg.addincompletedataforhtmlexport() | arg.importdesktopcontacts()),
-                               arg.importdesktopcontacts(), arg.autolimitdates(),  arg.importstickers(), arg.setselfid()))
+                               arg.importdesktopcontacts(), arg.autolimitdates(),  arg.importstickers(),
+                               arg.setselfid(), arg.targetisdummy()))
       return 1;
     MEMINFO("After importfromdesktop");
   }
@@ -405,7 +436,8 @@ int main(int argc, char *argv[])
 
     if (!sb->importFromPlaintextBackup(ptdb, arg.skipmessagereorder(), arg.mapxmlcontacts(), arg.limittodates(),
                                        arg.selectxmlchats(), arg.addincompletedataforhtmlexport(),
-                                       arg.xmlmarkdelivered(), arg.xmlmarkread(), arg.autolimitdates(), arg.setselfid()))
+                                       arg.xmlmarkdelivered(), arg.xmlmarkread(), arg.autolimitdates(), arg.setselfid(),
+                                       arg.targetisdummy()))
       return 1;
   }
 
@@ -472,20 +504,25 @@ int main(int argc, char *argv[])
   //   if (!sb->importWAChat(arg.importwachat(), arg.setwatimefmt(), arg.setselfid()))
   //     return 1;
 
+  if (!arg.setchatcolors().empty())
+    if (!sb->setChatColors(arg.setchatcolors()))
+      return 1;
+
+
   if (!arg.runsqlquery().empty())
     for (unsigned int i = 0; i < arg.runsqlquery().size(); ++i)
-      sb->runQuery(arg.runsqlquery()[i], false);
+      sb->runQuery(arg.runsqlquery()[i], arg.querymode());
 
   if (!arg.runprettysqlquery().empty())
     for (unsigned int i = 0; i < arg.runprettysqlquery().size(); ++i)
-      sb->runQuery(arg.runprettysqlquery()[i], true);
+      sb->runQuery(arg.runprettysqlquery()[i], "pretty");
 
   if (!arg.exporthtml().empty())
     if (!sb->exportHtml(arg.exporthtml(), limittothreads, arg.limittodates(), arg.split_by(), (arg.split_bool() ? arg.split() : -1),
                         arg.setselfid(), arg.includecalllog(), arg.searchpage(), arg.stickerpacks(), arg.migratedb(), arg.overwrite(),
                         arg.append(), arg.light(), arg.themeswitching(), arg.addexportdetails(), arg.includeblockedlist(),
                         arg.includefullcontactlist(), arg.includesettings(), arg.includereceipts(), arg.originalfilenames(),
-                        arg.linkify(), arg.chatfolders()))
+                        arg.linkify(), arg.chatfolders(), arg.compactfilenames(), arg.htmlignoremediatypes()))
       return 1;
 
   if (!arg.exporttxt().empty())
@@ -510,13 +547,13 @@ int main(int argc, char *argv[])
   //   sb->hhenkel(arg.hhenkel());
   // }
 
-  // // temporary, to switch sender and recipient in single one-to-one conversation INCOMPLETE
-  // if (arg.hiperfall() != -1)
-  //   if (!sb->hiperfall(arg.hiperfall(), arg.setselfid()))
-  //   {
-  //     std::cout << "Some error occurred..." << std::endl;
-  //     return 1;
-  //   }
+  // temporary, to switch sender and recipient in single one-to-one conversation INCOMPLETE
+  if (arg.hiperfall() != -1)
+    if (!sb->hiperfall(arg.hiperfall(), arg.setselfid()))
+    {
+      std::cout << "Some error occurred..." << std::endl;
+      return 1;
+    }
 
   // // temporary, to import messages from truncated database into older, but complete database
   // if (!arg.sleepyh34d().empty())
@@ -528,6 +565,16 @@ int main(int argc, char *argv[])
   //   }
   // }
 
+  // temporary,
+  if (arg.arc() != -1)
+  {
+    if (!sb->arc(arg.arc(), arg.setselfid()))
+    {
+      Logger::error("Failed somehow");
+      return 1;
+    }
+  }
+
   // // temporary, to investigate #95
   // if (!arg.carowit_1().empty())
   //   return sb->carowit(arg.carowit_1(), arg.carowit_2());
@@ -566,8 +613,15 @@ int main(int argc, char *argv[])
       return 1;
     }
 
+  // if (arg.migrate_to_191_CUSTOM())
+  //   if (!sb->migrate_to_191_CUSTOM(arg.setselfid()))
+  //   {
+  //     Logger::error("Migration failed");
+  //     return 1;
+  //   }
+
   if (arg.migrate_to_191())
-    if (!sb->migrate_to_191())
+    if (!sb->migrate_to_191(arg.setselfid()))
     {
       Logger::error("Migration failed");
       return 1;
