@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2023-2024  Selwin van Dijk
+  Copyright (C) 2023-2025  Selwin van Dijk
 
   This file is part of signalbackup-tools.
 
@@ -25,6 +25,7 @@
 
 #include "signalbackup.ih"
 
+#include "../scopeguard/scopeguard.h"
 #include <cerrno>
 
 bool SignalBackup::exportHtml(std::string const &directory, std::vector<long long int> const &limittothreads,
@@ -32,36 +33,16 @@ bool SignalBackup::exportHtml(std::string const &directory, std::vector<long lon
                               long long int split, std::string const &selfphone, bool calllog, bool searchpage,
                               bool stickerpacks, bool migrate, bool overwrite, bool append, bool lighttheme,
                               bool themeswitching, bool addexportdetails, bool blocked, bool fullcontacts,
-                              bool settings, bool receipts, bool originalfilenames, bool linkify, bool chatfolders)
+                              bool settings, bool receipts, bool originalfilenames, bool linkify, bool chatfolders,
+                              bool compact, std::vector<std::string> const &ignoremediatypes)
 {
   Logger::message("Starting HTML export to '", directory, "'");
 
+  // v170 and above should work. Anything below will first migrate (I believe anything down to ~23 should more or less work)
   bool databasemigrated = false;
   MemSqliteDB backup_database;
-
-  // >= 168 will work already? (not sure if 168 and 169 were ever in production, I don't have them at least)
-  if (d_databaseversion == 167)
+  if (d_databaseversion < 170 || migrate)
   {
-    SqliteDB::copyDb(d_database, backup_database);
-    if (!migrateDatabase(167, 170))
-    {
-      Logger::error("Failed to migrate currently unsupported database version (", d_databaseversion, ")."
-                    " Please upgrade your database");
-      SqliteDB::copyDb(backup_database, d_database);
-      return false;
-    }
-    else
-      databasemigrated = true;
-  }
-  else if (d_databaseversion < 167)
-  {
-    if (!migrate)
-    {
-      Logger::error("Currently unsupported database version (", d_databaseversion, ").");
-      Logger::error_indent("Please upgrade your database or append the `--migratedb' option to attempt to");
-      Logger::error_indent("migrate this database to a supported version.");
-      return false;
-    }
     SqliteDB::copyDb(d_database, backup_database);
     if (!migrateDatabase(d_databaseversion, 170)) // migrate == TRUE, but migration fails
     {
@@ -70,20 +51,16 @@ bool SignalBackup::exportHtml(std::string const &directory, std::vector<long lon
       SqliteDB::copyDb(backup_database, d_database);
       return false;
     }
-    else
-      databasemigrated = true;
+    databasemigrated = true;
   }
+  ScopeGuard restore_migrated_database([&]() { if (databasemigrated) SqliteDB::copyDb(backup_database, d_database); });
 
   if (originalfilenames && append) [[unlikely]]
     Logger::warning("Options 'originalfilenames' and 'append' are incompatible");
 
   // // check if dir exists, create if not
   if (!prepareOutputDirectory(directory, overwrite, !originalfilenames /*allowappend only allowed when not using original filenames*/, append))
-  {
-    if (databasemigrated)
-      SqliteDB::copyDb(backup_database, d_database);
     return false;
-  }
 
   // check and warn about selfid & note-to-self thread
   long long int note_to_self_thread_id = -1;
@@ -156,10 +133,13 @@ bool SignalBackup::exportHtml(std::string const &directory, std::vector<long lon
   SqliteDB::QueryResults search_idx_results;
   std::ofstream searchidx;
   bool searchidx_write_started = false;
+  long long int searchidx_page_idx = 0;
+  std::map<std::string, long long int> searchidx_page_idx_map;
+
   // start search index page
   if (searchpage)
   {
-    searchidx.open(directory + "/" + "searchidx.js", std::ios_base::binary);
+    searchidx.open(WIN_LONGPATH(directory + "/" + "searchidx.js"), std::ios_base::binary);
     if (!searchidx.is_open())
     {
       Logger::error("Failed to open 'searchidx.js' for writing");
@@ -277,6 +257,14 @@ bool SignalBackup::exportHtml(std::string const &directory, std::vector<long lon
 
     bool is_note_to_self = (t == note_to_self_thread_id);
 
+    // check if this is releasechannel:
+    bool is_releasechannel = false;
+    for (auto const &skv : d_keyvalueframes)
+      if (skv->key() == "releasechannel.recipient_id")
+        is_releasechannel =
+          (t == d_database.getSingleResultAs<long long int>("SELECT _id FROM thread WHERE " + d_thread_recipient_id + " = ?",
+                                                            bepaald::toNumber<int>(skv->value()), -1));
+
     // get recipient_id for thread;
     SqliteDB::QueryResults recid;
     long long int thread_recipient_id = -1;
@@ -297,7 +285,8 @@ bool SignalBackup::exportHtml(std::string const &directory, std::vector<long lon
     // now get all messages
     SqliteDB::QueryResults messages;
     d_database.exec("SELECT "s
-                    "_id, " + d_mms_recipient_id + ", body, "
+                    "_id, " + d_mms_recipient_id + ", "
+                    + (d_database.tableContainsColumn(d_mms_table, "to_recipient_id") ? "to_recipient_id" : "-1") +  " AS to_recipient_id, body, "
                     "MIN(date_received, " + d_mms_date_sent + ") AS bubble_date, "
                     "date_received, " + d_mms_date_sent + ", " + d_mms_type + ", "
                     + (!periodsplitformat.empty() ? "strftime('" + periodsplitformat + "', IFNULL(date_received, 0) / 1000, 'unixepoch', 'localtime')" : "''") + " AS periodsplit, "
@@ -310,6 +299,7 @@ bool SignalBackup::exportHtml(std::string const &directory, std::vector<long lon
                     + (d_database.tableContainsColumn(d_mms_table, "parent_story_id") ? "parent_story_id, " : "") +
                     + (d_database.tableContainsColumn(d_mms_table, "message_extras") ? "message_extras, " : "") +
                     + (d_database.tableContainsColumn(d_mms_table, "receipt_timestamp") ? "receipt_timestamp, " : "-1 AS receipt_timestamp, ") + // introduced in 117
+                    "json_extract(link_previews, '$[0].url') AS link_preview_url, "
                     "json_extract(link_previews, '$[0].title') AS link_preview_title, "
                     "json_extract(link_previews, '$[0].description') AS link_preview_description "
                     "FROM " + d_mms_table + " "
@@ -348,26 +338,22 @@ bool SignalBackup::exportHtml(std::string const &directory, std::vector<long lon
       continue;
     }
 
-    std::string threaddir = (is_note_to_self ? "Note to self (_id"s + bepaald::toString(thread_id) + ")"
-                             : sanitizeFilename(recipient_info[thread_recipient_id].display_name + " (_id" + bepaald::toString(thread_id) + ")"));
-
-    //if (!append)
-    //  makeFilenameUnique(directory, &threaddir);
+    std::string basethreaddir(is_note_to_self ? "Note to Self" : recipient_info[thread_recipient_id].display_name);
+    WIN_LIMIT_FILENAME_LENGTH(basethreaddir);
+    std::string threaddir(sanitizeFilename(basethreaddir) + " (_id"s + bepaald::toString(thread_id) + ")");
+    if (compact) [[unlikely]]
+      threaddir = "id" + bepaald::toString(thread_id);
 
     if (bepaald::fileOrDirExists(directory + "/" + threaddir))
     {
       if (!bepaald::isDir(directory + "/" + threaddir))
       {
         Logger::error("dir is regular file");
-        if (databasemigrated)
-          SqliteDB::copyDb(backup_database, d_database);
         return false;
       }
       if (!append && !overwrite) // should be impossible at this point....
       {
         Logger::error("Refusing to overwrite existing directory");
-        if (databasemigrated)
-          SqliteDB::copyDb(backup_database, d_database);
         return false;
       }
     }
@@ -385,8 +371,6 @@ bool SignalBackup::exportHtml(std::string const &directory, std::vector<long lon
           Logger::message(" Filesize: ", d_fd->total());
         }
       }
-      if (databasemigrated)
-        SqliteDB::copyDb(backup_database, d_database);
       return false;
     }
 
@@ -417,21 +401,29 @@ bool SignalBackup::exportHtml(std::string const &directory, std::vector<long lon
       std::string previous_period_split_string(messages(messagecount, "periodsplit"));
       std::string previous_day_change;
       // create output-file
-      std::string raw_base_filename = (is_note_to_self ? "Note to self" : recipient_info[thread_recipient_id].display_name);
-      std::string filename = sanitizeFilename(raw_base_filename + (pagenumber > 0 ? "_" + bepaald::toString(pagenumber) : "") + ".html");
-      std::ofstream htmloutput(directory + "/" + threaddir + "/" + filename, std::ios_base::binary);
+      std::string raw_base_filename = (is_note_to_self ? "Note to Self" : recipient_info[thread_recipient_id].display_name);
+      WIN_LIMIT_FILENAME_LENGTH(raw_base_filename);
+      std::string sanitized_base_filename(sanitizeFilename(raw_base_filename));
+      std::string filename(sanitized_base_filename + (pagenumber > 0 ? "_" + bepaald::toString(pagenumber) : "") + ".html");
+      if (compact) [[unlikely]]
+      {
+        sanitized_base_filename.clear();
+        filename = bepaald::toString(pagenumber) + ".html";
+      }
+
+      WIN_CHECK_PATH_LENGTH(directory + "/" + threaddir + "/" + filename);
+
+      std::ofstream htmloutput(WIN_LONGPATH(directory + "/" + threaddir + "/" + filename), std::ios_base::binary);
       if (!htmloutput.is_open())
       {
-        Logger::error("Failed to open '", directory, "/", threaddir, "/", filename, " for writing.");
-        if (databasemigrated)
-          SqliteDB::copyDb(backup_database, d_database);
+        Logger::error("Failed to open '", directory, "/", threaddir, "/", filename, "' for writing.");
         return false;
       }
 
       // create start of html (css, head, start of body
       HTMLwriteStart(htmloutput, thread_recipient_id, directory, threaddir, isgroup, is_note_to_self,
-                     all_recipients_ids, &recipient_info, &written_avatars, overwrite, append,
-                     lighttheme, themeswitching, searchpage, addexportdetails);
+                     is_releasechannel, all_recipients_ids, &recipient_info, &written_avatars, overwrite,
+                     append, lighttheme, themeswitching, searchpage, addexportdetails);
       while (messagecount < (max_msg_per_page * (pagenumber + 1)) &&
              messages(messagecount, "periodsplit") == previous_period_split_string)
       {
@@ -523,17 +515,31 @@ bool SignalBackup::exportHtml(std::string const &directory, std::vector<long lon
         IconType icon = IconType::NONE;
         if (Types::isStatusMessage(type))
         {
+          // identityVerified and identityDefault ("You marked your safety number with XXX (un)verified") are
+          // a little different from other (nongroup) statusmessages, in that the name to be filled in is not
+          // the FROM_recipient_id (as is the case with others: "XXX reset the secure session", "XXX set the
+          // disappearing message timer to ..."), but the TO_recipient. For these message FROM is always self
+          // ("YOU set YOUR safety number..."), but we need the TO_id to fill in the name.
+          // Note this is even true in group threads, where outgoing messages are usually always from you and
+          // to the group, but not for these two status messages.
+          // But all of the above is only true for (newer) databases that _have_ from_ and to_recipient_ids,
+          // in older databases the target-name is recipient_id in all cases...
+          long long int target_rid = msg_recipient_id;
+          if ((Types::isIdentityVerified(type) || Types::isIdentityDefault(type)) &&
+              messages.valueAsInt(messagecount, "to_recipient_id") != -1) [[unlikely]]
+            target_rid = messages.valueAsInt(messagecount, "to_recipient_id");
+
           // decode from body if (body not empty) OR (message_extras not available)
           if (!body.empty() ||
               !(d_database.tableContainsColumn(d_mms_table, "message_extras") &&
                 messages.valueHasType<std::pair<std::shared_ptr<unsigned char []>, size_t>>(messagecount, "message_extras")))
             body = decodeStatusMessage(body, messages.getValueAs<long long int>(messagecount, "expires_in"),
-                                       type, getRecipientInfoFromMap(&recipient_info, msg_recipient_id).display_name, &icon);
+                                       type, getRecipientInfoFromMap(&recipient_info, target_rid).display_name, &icon);
           else if (d_database.tableContainsColumn(d_mms_table, "message_extras") &&
                    messages.valueHasType<std::pair<std::shared_ptr<unsigned char []>, size_t>>(messagecount, "message_extras"))
             body = decodeStatusMessage(messages.getValueAs<std::pair<std::shared_ptr<unsigned char []>, size_t>>(messagecount, "message_extras"),
                                        messages.getValueAs<long long int>(messagecount, "expires_in"), type,
-                                       getRecipientInfoFromMap(&recipient_info, msg_recipient_id).display_name, &icon);
+                                       getRecipientInfoFromMap(&recipient_info, target_rid).display_name, &icon);
         }
 
         // prep body (scan emoji? -> in <span>) and handle mentions...
@@ -625,16 +631,22 @@ bool SignalBackup::exportHtml(std::string const &directory, std::vector<long lon
                                   directory,
                                   threaddir,
                                   filename,
-                                  messages(messagecount, "link_preview_title"),
+                                  messages(messagecount, "link_preview_url"),
+                                  messages(messagecount, "link_preview_title") ,
                                   messages(messagecount, "link_preview_description"),
                                   shared_contacts,
 
                                   icon
           });
-        HTMLwriteMessage(htmloutput, msg_info, &recipient_info, searchpage, receipts);
+        HTMLwriteMessage(htmloutput, msg_info, &recipient_info, searchpage, receipts, ignoremediatypes);
 
         if (searchpage && (!Types::isStatusMessage(msg_info.type) && !msg_info.body.empty()))
         {
+          if (auto it = searchidx_page_idx_map.find(msg_info.threaddir + "/" + sanitized_base_filename); it != searchidx_page_idx_map.end())
+            searchidx_page_idx = it->second;
+          else
+            searchidx_page_idx_map.emplace(msg_info.threaddir + "/" + sanitized_base_filename, ++searchidx_page_idx);
+
           // because the body is already escaped for html at this point, we get it fresh from database (and have sqlite do the json formatting)
           if (!d_database.exec("SELECT json_object("
                                "'id', " + d_mms_table + "._id, "
@@ -642,9 +654,11 @@ bool SignalBackup::exportHtml(std::string const &directory, std::vector<long lon
                                "'f', " + d_mms_table + "." + d_mms_recipient_id + ", "
                                "'tr', thread." + d_thread_recipient_id + ", "
                                "'o', (" + d_mms_table + "." + d_mms_type + " & 0x1F) IN (2,11,21,22,23,24,25,26), "
-                               "'d', (" + d_mms_table + ".date_received / 1000 - 1404165600), " // loose the last three digits (miliseconds, they are never displayed anyway).
-                                                                                                // subtract "2014-07-01". Signals initial release was 2014-07-29, negative numebrs should work otherwise anyway.
-                               "'p', " + "SUBSTR(\"" + msg_info.threaddir + "/" + msg_info.filename + "\", 1, LENGTH(\"" + msg_info.threaddir + "/" + msg_info.filename + "\") - 5)" + ") AS line, " // all pages end in ".html", slice it off
+                               "'d', (" + d_mms_table + ".date_received / 1000 - 1404165600), " // lose the last three digits (miliseconds, they are never displayed anyway).
+                                                                                                // subtract "2014-07-01". Signals initial release was 2014-07-29, negative
+                                                                                                // numbers should work otherwise anyway.
+                               "'p', ?, "
+                               "'n', ?) AS line,"
                                + d_part_table + "._id AS rowid, " +
                                (d_database.tableContainsColumn(d_part_table, "unique_id") ?
                                 d_part_table + ".unique_id AS uniqueid" : "-1 AS uniqueid") +
@@ -652,7 +666,7 @@ bool SignalBackup::exportHtml(std::string const &directory, std::vector<long lon
                                "LEFT JOIN thread ON thread._id IS " + d_mms_table + ".thread_id "
                                "LEFT JOIN " + d_part_table + " ON " + d_part_table + "." + d_part_mid + " IS " + d_mms_table + "._id AND " + d_part_table + "." + d_part_ct + " = 'text/x-signal-plain' AND " + d_part_table + ".quote = 0 "
                                "WHERE " + d_mms_table + "._id = ?",
-                               msg_info.msg_id, &search_idx_results) ||
+                               {searchidx_page_idx, pagenumber, msg_info.msg_id}, &search_idx_results) ||
               search_idx_results.rows() < 1) [[unlikely]]
           {
             Logger::warning("Search_idx query failed or no results");
@@ -732,28 +746,28 @@ bool SignalBackup::exportHtml(std::string const &directory, std::vector<long lon
 
       if (totalpages > 1)
       {
-        std::string sanitized_filename = sanitizeFilename(raw_base_filename);
+        std::string sanitized_filename(sanitized_base_filename);
         HTMLescapeUrl(&sanitized_filename);
         htmloutput << "      <div class=\"conversation-link conversation-link-left\">" << '\n';
         htmloutput << "        <div title=\"First page\">" << '\n';
-        htmloutput << "          <a href=\"" << sanitized_filename << ".html" << "\">" << '\n';
+        htmloutput << "          <a href=\"" << (compact ? "0" : sanitized_filename) << ".html" << "\">" << '\n';
         htmloutput << "            <div class=\"menu-icon nav-max" << (pagenumber > 0 ? "" : " nav-disabled") << "\"></div>" << '\n';
         htmloutput << "          </a>" << '\n';
         htmloutput << "        </div>" << '\n';
         htmloutput << "        <div title=\"Previous page\">" << '\n';
-        htmloutput << "          <a href=\"" << sanitized_filename << (pagenumber - 1 > 0 ? ("_" + bepaald::toString(pagenumber - 1)) : "") << ".html" << "\">" << '\n';
+        htmloutput << "          <a href=\"" << sanitized_filename << (compact ? bepaald::toString(pagenumber - 1) : (pagenumber - 1 > 0 ? ("_" + bepaald::toString(pagenumber - 1)) : "")) << ".html" << "\">" << '\n';
         htmloutput << "            <div class=\"menu-icon nav-one" << (pagenumber > 0 ? "" : " nav-disabled") << "\"></div>" << '\n';
         htmloutput << "          </a>" << '\n';
         htmloutput << "        </div>" << '\n';
         htmloutput << "      </div>" << '\n';
         htmloutput << "      <div class=\"conversation-link conversation-link-right\">" << '\n';
         htmloutput << "        <div title=\"Next page\">" << '\n';
-        htmloutput << "          <a href=\"" << sanitized_filename << "_" << (pagenumber + 1 <= totalpages - 1 ?  bepaald::toString(pagenumber + 1) : bepaald::toString(totalpages - 1)) << ".html" << "\">" << '\n';
+        htmloutput << "          <a href=\"" << sanitized_filename << (compact ? "" : "_") << (pagenumber + 1 <= totalpages - 1 ?  bepaald::toString(pagenumber + 1) : bepaald::toString(totalpages - 1)) << ".html" << "\">" << '\n';
         htmloutput << "            <div class=\"menu-icon nav-one nav-fwd" << (pagenumber < totalpages - 1 ? "" : " nav-disabled") << "\"></div>" << '\n';
         htmloutput << "          </a>" << '\n';
         htmloutput << "        </div>" << '\n';
         htmloutput << "        <div title=\"Last page\">" << '\n';
-        htmloutput << "          <a href=\"" << sanitized_filename << "_" << bepaald::toString(totalpages - 1) << ".html" << "\">" << '\n';
+        htmloutput << "          <a href=\"" << sanitized_filename << (compact ? "" : "_") << bepaald::toString(totalpages - 1) << ".html" << "\">" << '\n';
         htmloutput << "            <div class=\"menu-icon nav-max nav-fwd" << (pagenumber < totalpages - 1 ? "" : " nav-disabled") << "\"></div>" << '\n';
         htmloutput << "          </a>" << '\n';
         htmloutput << "        </div>" << '\n';
@@ -856,13 +870,26 @@ bool SignalBackup::exportHtml(std::string const &directory, std::vector<long lon
       std::string line = d_database.getSingleResultAs<std::string>("SELECT json_object('_id', ?, 'display_name', ?)", {r->first, r->second.display_name}, std::string());
       if (line.empty()) [[unlikely]]
         continue;
-
       searchidx << "  " << line;
       if (std::next(r) != recipient_info.end()) [[likely]]
         searchidx << "," << std::endl;
       else
         searchidx << std::endl << "];" << std::endl;
     }
+
+    // write page info:
+    searchidx << "page_idx = [" << std::endl;
+    for (auto pi = searchidx_page_idx_map.begin() ; pi != searchidx_page_idx_map.end(); ++pi)
+    {
+      std::string line = d_database.getSingleResultAs<std::string>("SELECT json_object('_id', ?, 'bn', ?)", {pi->second, pi->first}, std::string());
+      if (line.empty()) [[unlikely]]
+        continue;
+      searchidx << "  " << line;
+      if (std::next(pi) != searchidx_page_idx_map.end()) [[likely]]
+        searchidx << "," << std::endl;
+      else
+        searchidx << std::endl << "];" << std::endl;
+    }
   }
 
   // write chat folders
@@ -870,7 +897,7 @@ bool SignalBackup::exportHtml(std::string const &directory, std::vector<long lon
   std::set_difference(threads.begin(), threads.end(),
                       excludethreads.begin(), excludethreads.end(),
                       std::back_inserter(indexedthreads));
-  std::vector<std::pair<std::string, std::string>> chatfolders_list; // {chatfolder name, filename(link)}
+  std::vector<std::tuple<long long int, std::string, std::string>> chatfolders_list; // {_id, chatfolder name, filename(link)}
   if (chatfolders && d_database.containsTable("chat_folder"))
   {
     // get all folders
@@ -893,7 +920,7 @@ bool SignalBackup::exportHtml(std::string const &directory, std::vector<long lon
       for (unsigned int i = 0; i < cf_results.rows(); ++i)
       {
         std::string filename = "chatfolder_" + cf_results(i, "_id") + "_" + cf_results(i, "name");
-        chatfolders_list.emplace_back(cf_results(i, "name"), filename);
+        chatfolders_list.emplace_back(cf_results.valueAsInt(i, "_id"), cf_results(i, "name"), filename);
       }
 
       for (unsigned int i = 0; i < cf_results.rows(); ++i)
@@ -963,38 +990,36 @@ bool SignalBackup::exportHtml(std::string const &directory, std::vector<long lon
         std::string filename = "chatfolder_" + cf_results(i, "_id") + "_" + cf_results(i, "name");
         HTMLwriteChatFolder(chatfolder_threads, maxdate, directory, filename, &recipient_info, note_to_self_thread_id,
                             calllog, searchpage, stickerpacks, blocked, fullcontacts, settings, overwrite,
-                            append, lighttheme, themeswitching, exportdetails_html, cf_results.valueAsInt(i, "_id"));
+                            append, lighttheme, themeswitching, exportdetails_html, cf_results.valueAsInt(i, "_id"),
+                            chatfolders_list, compact);
       }
     }
   }
   HTMLwriteIndex(indexedthreads, maxdate, directory, &recipient_info, note_to_self_thread_id,
                  calllog, searchpage, stickerpacks, blocked, fullcontacts, settings, overwrite,
-                 append, lighttheme, themeswitching, exportdetails_html, chatfolders_list);
+                 append, lighttheme, themeswitching, exportdetails_html, chatfolders_list, compact);
 
   if (calllog)
     HTMLwriteCallLog(threads, directory, datewhereclausecalllog, &recipient_info, note_to_self_thread_id,
-                     overwrite, append, lighttheme, themeswitching, exportdetails_html);
+                     overwrite, append, lighttheme, themeswitching, exportdetails_html, compact);
 
   if (searchpage)
-    HTMLwriteSearchpage(directory, lighttheme, themeswitching);
+    HTMLwriteSearchpage(directory, lighttheme, themeswitching, compact);
 
   if (stickerpacks)
     HTMLwriteStickerpacks(directory, overwrite, append, lighttheme, themeswitching, exportdetails_html);
 
   if (blocked)
-    HTMLwriteBlockedlist(directory, &recipient_info, overwrite, append, lighttheme, themeswitching, exportdetails_html);
+    HTMLwriteBlockedlist(directory, &recipient_info, overwrite, append, lighttheme, themeswitching,
+                         exportdetails_html, compact);
 
   if (fullcontacts)
-    HTMLwriteFullContacts(directory, &recipient_info, overwrite, append, lighttheme, themeswitching, exportdetails_html);
+    HTMLwriteFullContacts(directory, &recipient_info, overwrite, append, lighttheme, themeswitching,
+                          exportdetails_html, compact);
 
   if (settings)
     HTMLwriteSettings(directory, overwrite, append, lighttheme, themeswitching, exportdetails_html);
 
   Logger::message("All done!");
-  if (databasemigrated)
-  {
-    Logger::message("restoring migrated database...");
-    SqliteDB::copyDb(backup_database, d_database);
-  }
   return true;
 }
