@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2019-2024  Selwin van Dijk
+  Copyright (C) 2019-2025  Selwin van Dijk
 
   This file is part of signalbackup-tools.
 
@@ -46,6 +46,31 @@
 #include <string>
 #include <algorithm>
 #include <regex>
+#include <array>
+
+#if defined WIN32 || MINGW
+// Windows has all sorts of issues with 'long' paths. While this tool can (probably)
+// work around that (see WIN_LONGPATH in common_filesystem), it does not help much
+// as many Windows programs can then still not open the resulting files with their
+// long paths. Instead we limit the file length of files written to some max length
+//
+// In Signal currently group titles are limited to 32 characters, while contact names
+// are max 53 (26 first name + space + 26 last name) (historically, longer group
+// titles have been possible).
+// maybe this needs to be done smarter, making sure to only truncate at character
+// boundaries (not in the middle of multibyte ones...
+#define MAXFILELENGTH 54
+//#define WIN_LIMIT_FILENAME_LENGTH(str) if (str.size() > MAXFILELENGTH) [[unlikely]]  str.resize(MAXFILELENGTH);
+#define WIN_LIMIT_FILENAME_LENGTH(str)
+#else
+#define WIN_LIMIT_FILENAME_LENGTH(str)
+#endif
+
+#if defined WIN32 || MINGW
+#define WIN_CHECK_PATH_LENGTH(str) if (int pl = bepaald::abs_path_length(str); pl >= 260) Logger::warnOnce("Path length is " + bepaald::toString(pl) + ". This may be more than the allowed maximum path length on your platform. If you run into problems, use the option `--compactfilenames' to shorten the filenames this tool writes.", false, STRLEN("Path length is "));
+#else
+#define WIN_CHECK_PATH_LENGTH(str)
+#endif
 
 struct HTMLMessageInfo;
 struct Range;
@@ -77,6 +102,7 @@ class SignalBackup
   std::string d_thread_message_count;
   std::string d_thread_delivery_receipts;
   std::string d_thread_read_receipts;
+  std::string d_thread_pinned;
   std::string d_sms_date_received;
   std::string d_sms_recipient_id;
   std::string d_sms_recipient_device_id;
@@ -108,6 +134,7 @@ class SignalBackup
   // table/column names for desktop db
   std::string d_dt_c_uuid;
   std::string d_dt_m_sourceuuid;
+  std::string d_dt_s_uuid;
 
   std::vector<std::pair<std::string, DeepCopyingUniquePtr<AvatarFrame>>> d_avatars;
   std::map<std::pair<uint64_t, int64_t>, DeepCopyingUniquePtr<AttachmentFrame>> d_attachments; //maps <rowid,uniqueid> to attachment
@@ -126,7 +153,6 @@ class SignalBackup
   bool d_verbose;
   bool d_truncate;
   bool d_fulldecode;
-  std::set<std::string> d_warningsgiven;
   long long int d_selfid;
   std::string d_selfuuid;
 
@@ -182,12 +208,15 @@ class SignalBackup
     std::string phone;
     std::string username;
     long long int mute_until;           // -1 : n/a
+    bool blocked;
     long long int mention_setting;      // -1 : n/a
+    long long int message_expiration_time;
     long long int custom_notifications; // -1 : n/a
     std::string color; // "RRGGBB"
     std::string wall_light;
     std::string wall_dark;
     bool hasavatar;
+    bool verified;
   };
 
   static std::vector<DatabaseLink> const s_databaselinks;
@@ -196,6 +225,7 @@ class SignalBackup
   static std::unordered_set<char> const s_emoji_first_bytes;
   static unsigned int constexpr s_emoji_min_size = 2; // smallest emoji_unicode_size - 1
   static std::map<std::string, std::string> const s_html_colormap;
+  static std::array<std::pair<std::string, std::string>, 12> const s_html_random_colors;
   static std::regex const s_linkify_pattern;
 
  protected:
@@ -231,7 +261,7 @@ class SignalBackup
   inline void addEndFrame();
   bool mergeRecipients(std::vector<std::string> const &addresses);
   void mergeGroups(std::vector<std::string> const &groups);
-  inline void runQuery(std::string const &q, bool pretty = true) const;
+  inline void runQuery(std::string const &q, std::string const &mode = std::string()) const;
   void removeDoubles(long long int milliseconds = 0);
   inline std::vector<long long int> threadIds() const;
   bool importCSV(std::string const &file, std::map<std::string, std::string> const &fieldmap);
@@ -251,12 +281,16 @@ class SignalBackup
   bool importFromDesktop(std::unique_ptr<DesktopDatabase> const &dtdb, bool skipmessagereorder,
                          std::vector<std::string> const &dateranges, bool createmissingcontacts,
                          bool createcontacts_nowarn, bool autodates, bool importstickers,
-                         std::string const &selfphone);
+                         std::string const &selfphone, bool targetisdummy);
   bool importFromPlaintextBackup(std::unique_ptr<SignalPlaintextBackupDatabase> const &ptdb, bool skipmessagereorder,
                                  std::vector<std::pair<std::string, long long int>> const &initial_contactmap,
                                  std::vector<std::string> const &daterangelist, std::vector<std::string> const &chats,
                                  bool createmissingcontacts, bool markdelivered, bool markread, bool autodates,
-                                 std::string const &selfphone);
+                                 std::string const &selfphone, bool targetisdummy);
+  long long int ptCreateRecipient(std::unique_ptr<SignalPlaintextBackupDatabase> const &ptdb,
+                                  std::map<std::string, long long int> *contactmap,
+                                  bool *warned_createcontacts, std::string const &contact_name,
+                                  std::string const &address, bool isgroup) const;
   bool checkDbIntegrity(bool warn = false) const;
   bool exportHtml(std::string const &directory, std::vector<long long int> const &threads,
                   std::vector<std::string> const &dateranges, std::string const &splitby,
@@ -264,7 +298,7 @@ class SignalBackup
                   bool stickerpacks, bool migrate, bool overwrite, bool append, bool theme,
                   bool themeswitching, bool addexportdetails, bool blocked, bool fullcontacts,
                   bool settings, bool receipts, bool use_original_filenames, bool linkify,
-                  bool chatfolders);
+                  bool chatfolders, bool compact, std::vector<std::string> const &ignoremediatypes);
   bool exportTxt(std::string const &directory, std::vector<long long int> const &threads,
                  std::vector<std::string> const &dateranges, std::string const &selfid, bool migrate, bool overwrite);
   bool findRecipient(long long int id) const;
@@ -277,19 +311,22 @@ class SignalBackup
                           std::vector<std::pair<std::string, long long int>> contactmap,
                           std::vector<std::string> const &inhibitmapping, bool prependforwarded,
                           bool skipmessagereorder, bool markdelivered, bool markread, std::string const &selfphone);
+  bool setChatColors(std::vector<std::pair<long long int, std::string>> const &colorlist);
 
   /* CUSTOMS */
   //bool hhenkel(std::string const &);
   //bool sleepyh34d(std::string const &truncatedbackup, std::string const &pwd);
-  //bool hiperfall(uint64_t t_id, std::string const &selfid);
+  bool hiperfall(uint64_t t_id, std::string const &selfid);
   void scanMissingAttachments() const;
   //void devCustom() const;
   //bool carowit(std::string const &sourcefile, std::string const &sourcepw) const;
   bool custom_hugogithubs();
+  bool arc(long long int tid, std::string const &selfphone);
 
   // for bug 233/13034
-  bool migrate_to_191();
+  bool migrate_to_191(std::string const &selfphone);
   /* CUSTOMS */
+  //bool migrate_to_191_CUSTOM(std::string const &selfphone);
 
  protected:
   [[nodiscard]] bool exportBackupToFile(std::string const &filename, std::string const &passphrase,
@@ -381,18 +418,20 @@ class SignalBackup
                   std::string const &returnfield = std::string(), std::any *returnvalue = nullptr) const;
   bool dtInsertAttachments(long long int mms_id, long long int unique_id, int numattachments, long long int haspreviews,
                            long long int rowid, SqliteDB const &ddb, std::string const &where,
-                           std::string const &databasedir, bool isquote, bool issticker);
+                           std::string const &databasedir, bool isquote, bool issticker, bool targetisdummy);
   bool handleDTCallTypeMessage(SqliteDB const &ddb, std::string const &callid, long long int rowid, long long int ttid, long long int address, bool insertincompletedataforexport) const;
-  void handleDTGroupChangeMessage(SqliteDB const &ddb, long long int rowid, long long int thread_id, long long int address,
-                                  long long int date, std::map<long long int, long long int> *adjusted_timestamps, std::map<std::string, long long int> *savedmap, bool istimermessage) const;
+  bool handleDTGroupChangeMessage(SqliteDB const &ddb, long long int rowid, long long int thread_id, long long int address,
+                                  long long int date, std::map<long long int, long long int> *adjusted_timestamps, std::map<std::string, long long int> *savedmap,
+                                  std::string const &databasedir, bool istimermessage, bool createcontacts, bool create_valid_contacts, bool *warn);
   bool handleDTExpirationChangeMessage(SqliteDB const &ddb, long long int rowid, long long int ttid, long long int sent_at, long long int address) const;
   bool handleDTGroupV1Migration(SqliteDB const &ddb, long long int rowid, long long int thread_id, long long int timestamp,
                                 long long int address, std::map<std::string, long long int> *savedmap,
-                                bool createcontacts, std::string const &databasedir, bool *warn);
+                                bool createcontacts, std::string const &databasedir, bool create_valid_contacts, bool *warn);
   void getDTReactions(SqliteDB const &ddb, long long int rowid, long long int numreactions,
                       std::vector<std::vector<std::string>> *reactions) const;
-  void insertReactions(long long int message_id, std::vector<std::vector<std::string>> const &reactions, bool mms,
-                       std::map<std::string, long long int> *savedmap) const;
+  void dtInsertReactions(SqliteDB const &ddb, long long int message_id, std::vector<std::vector<std::string>> const &reactions, bool mms,
+                         std::map<std::string, long long int> *savedmap, std::string const &databasedir, bool createcontacts,
+                         bool create_valid_contacts);
   long long int getRecipientIdFromUuidMapped(std::string const &uuid, std::map<std::string, long long int> *savedmap,
                                              bool suppresswarning = false) const;
   long long int getRecipientIdFromPhoneMapped(std::string const &phone, std::map<std::string, long long int> *savedmap,
@@ -400,20 +439,25 @@ class SignalBackup
   inline std::string getNameFromUuid(std::string const &uuid) const;
   std::string getNameFromRecipientId(long long int id) const;
   void dtSetMessageDeliveryReceipts(SqliteDB const &ddb, long long int rowid, std::map<std::string, long long int> *savedmap,
-                                    std::string const &databasedir, bool createcontacts, long long int msg_id, bool is_mms, bool isgroup, bool *warn);
+                                    std::string const &databasedir, bool createcontacts, long long int msg_id, bool is_mms, bool isgroup,
+                                    bool create_valid_contacts, bool *warn);
   bool HTMLwriteStart(std::ofstream &file, long long int thread_recipient_id, std::string const &directory,
-                      std::string const &threaddir, bool isgroup, bool isnotetoself, std::set<long long int> const &recipients,
-                      std::map<long long int, RecipientInfo> *recipientinfo,
+                      std::string const &threaddir, bool isgroup, bool isnotetoself, bool isreleasechannel,
+                      std::set<long long int> const &recipients, std::map<long long int, RecipientInfo> *recipientinfo,
                       std::map<long long int, std::string> *written_avatars, bool overwrite, bool append,
                       bool light, bool themeswitching, bool searchpage, bool exportdetails) const;
   void HTMLwriteAttachmentDiv(std::ofstream &htmloutput, SqliteDB::QueryResults const &attachment_results, int indent,
                               std::string const &directory, std::string const &threaddir, bool use_original_filenames,
-                              bool is_image_preview, bool overwrite, bool append) const;
+                              bool is_image_preview, bool overwrite, bool append, std::vector<std::string> const &ignoremediatypes) const;
+  void HTMLwriteCallLinkDiv(std::ofstream &htmloutput, int indent, std::string const &url, std::string const &title,
+                            std::string const &description/*, std::string const &directory, std::string const &threaddir,
+                                                              bool overwrite, bool append*/) const;
   void HTMLwriteSharedContactDiv(std::ofstream &htmloutput, std::string const &shared_contact, int indent,
                                  std::string const &directory, std::string const &threaddir,
                                  bool overwrite, bool append) const;
   bool HTMLwriteAttachment(std::string const &directory, std::string const &threaddir, long long int rowid,
-                           long long int uniqueid, std::string const &attachment_filename, bool overwrite, bool append) const;
+                           long long int uniqueid, std::string const &attachment_filename, long long int timestamp,
+                           bool overwrite, bool append) const;
   bool HTMLprepMsgBody(std::string *body, std::vector<std::tuple<long long int, long long int, long long int>> const &mentions,
                        std::map<long long int, RecipientInfo> *recipients_info, bool incoming,
                        std::pair<std::shared_ptr<unsigned char []>, size_t> const &brdata,
@@ -421,9 +465,10 @@ class SignalBackup
   std::string HTMLwriteAvatar(long long int recipient_id, std::string const &directory, std::string const &threaddir,
                               bool overwrite, bool append) const;
   void HTMLwriteMessage(std::ofstream &filt, HTMLMessageInfo const &msginfo, std::map<long long int, RecipientInfo> *recipientinfo,
-                        bool searchpage, bool writereceipts) const;
+                        bool searchpage, bool writereceipts, std::vector<std::string> const &ignoremediatypes) const;
   void HTMLwriteRevision(long long int msg_id, std::ofstream &filt, HTMLMessageInfo const &parent_info,
-                         std::map<long long int, RecipientInfo> *recipientinfo, bool linkify) const;
+                         std::map<long long int, RecipientInfo> *recipientinfo, bool linkify,
+                         std::vector<std::string> const &ignoremediatypes) const;
   void HTMLwriteMsgReceiptInfo(std::ofstream &htmloutput, std::map<long long int, RecipientInfo> *recipientinfo,
                                long long int message_id, bool isgroup, long long int read_count,
                                long long int delivered_count, long long int timestamp, int indent) const;
@@ -432,30 +477,31 @@ class SignalBackup
                              std::map<long long int, RecipientInfo> *recipient_info, long long int note_to_self_tid, bool calllog,
                              bool searchpage, bool stickerpacks, bool blocked, bool fullcontacts, bool settings,  bool overwrite,
                              bool append, bool light, bool themeswitching, std::string const &exportdetails,
-                             std::vector<std::pair<std::string, std::string>> const &chatfolders) const;
+                             std::vector<std::tuple<long long int, std::string, std::string>> const &chatfolders,
+                             bool compact) const;
   inline bool HTMLwriteChatFolder(std::vector<long long int> const &threads, long long int maxtimestamp, std::string const &directory,
                                   std::string const &basename,  std::map<long long int, RecipientInfo> *recipient_info, long long int note_to_self_tid,
                                   bool calllog, bool searchpage, bool stickerpacks, bool blocked, bool fullcontacts, bool settings,  bool overwrite,
-                                  bool append, bool light, bool themeswitching, std::string const &exportdetails, long long int chatfolderidx) const;
+                                  bool append, bool light, bool themeswitching, std::string const &exportdetails, long long int chatfolderidx,
+                                  std::vector<std::tuple<long long int, std::string, std::string>> const &chatfolders, bool compact) const;
 
   bool HTMLwriteIndexImpl(std::vector<long long int> const &threads, long long int maxtimestamp, std::string const &directory,
                           std::string const &basename,  std::map<long long int, RecipientInfo> *recipient_info, long long int note_to_self_tid,
                           bool calllog, bool searchpage, bool stickerpacks, bool blocked, bool fullcontacts, bool settings,  bool overwrite,
                           bool append, bool light, bool themeswitching, std::string const &exportdetails, long long int chatfolderidx,
-                          std::vector<std::pair<std::string, std::string>> const &chatfolders) const;
+                          std::vector<std::tuple<long long int, std::string, std::string>> const &chatfolders, bool compact) const;
 
-
-  void HTMLwriteSearchpage(std::string const &dir, bool light, bool themeswitching) const;
+  void HTMLwriteSearchpage(std::string const &dir, bool light, bool themeswitching, bool compact) const;
   void HTMLwriteCallLog(std::vector<long long int> const &threads, std::string const &directory,
                         std::string const &datewhereclause, std::map<long long int, RecipientInfo> *recipientinfo,
                         long long int notetoself_tid, bool overwrite, bool append, bool light, bool themeswitching,
-                        std::string const &exportdetails) const;
+                        std::string const &exportdetails, bool compact) const;
   bool HTMLwriteStickerpacks(std::string const &dir, bool overwrite, bool append, bool light, bool themeswitching,
                              std::string const &exportdetails) const;
-  bool HTMLwriteBlockedlist(std::string const &dir, std::map<long long int, RecipientInfo> *recipientinfo,
-                            bool overwrite, bool append, bool light, bool themeswitching, std::string const &exportdetails) const;
-  bool HTMLwriteFullContacts(std::string const &dir, std::map<long long int, RecipientInfo> *recipientinfo,
-                             bool overwrite, bool append, bool light, bool themeswitching, std::string const &exportdetails) const;
+  bool HTMLwriteBlockedlist(std::string const &dir, std::map<long long int, RecipientInfo> *recipientinfo, bool overwrite,
+                            bool append, bool light, bool themeswitching, std::string const &exportdetails, bool compact) const;
+  bool HTMLwriteFullContacts(std::string const &dir, std::map<long long int, RecipientInfo> *recipientinfo, bool overwrite,
+                             bool append, bool light, bool themeswitching, std::string const &exportdetails, bool compact) const;
   bool HTMLwriteSettings(std::string const &dir, bool overwrite, bool append, bool light,
                          bool themeswitching, std::string const &exportdetails) const;
   void HTMLescapeString(std::string *in, std::set<int> const *const positions_excluded_from_escape = nullptr) const;
@@ -479,16 +525,15 @@ class SignalBackup
   std::string utf8BytesToHexString(unsigned char const *const data, size_t data_size) const;
   inline std::string utf8BytesToHexString(std::shared_ptr<unsigned char[]> const &data, size_t data_size) const;
   inline std::string utf8BytesToHexString(std::string const &data) const;
-  RecipientInfo const &getRecipientInfoFromMap(std::map<long long int, RecipientInfo> *recipient_info, long long int rid) const;
+  inline RecipientInfo const &getRecipientInfoFromMap(std::map<long long int, RecipientInfo> *recipient_info, long long int rid) const;
   bool migrateDatabase(int from, int to) const;
   long long int dtCreateRecipient(SqliteDB const &ddb, std::string const &id, std::string const &phone, std::string const &gidb64,
                                   std::string const &databasedir, std::map<std::string, long long int> *recipient_info,
-                                  bool *warn);
+                                  bool create_valid_contacts, bool *warn);
   bool dtUpdateProfile(SqliteDB const &ddb, std::string const &dtid, long long int aid, std::string const &databasedir);
   bool dtSetAvatar(std::string const &avatarpath, std::string const &key, int64_t size, int version,
                    long long int rid, std::string const &databasedir);
   std::string dtSetSharedContactsJsonString(SqliteDB const &ddb, long long int rowid) const;
-  void warnOnce(std::string const &msg, bool error = false);
   void getGroupInfo(long long int rid, GroupInfo *groupinfo) const;
   std::pair<std::string, std::string> getCustomColor(std::pair<std::shared_ptr<unsigned char []>, size_t> const &colorproto) const;
   std::string HTMLprepLinkPreviewDescription(std::string const &in) const;
@@ -516,6 +561,7 @@ class SignalBackup
   long long int getRecipientIdFromField(std::string const &field, std::string const &value, bool withthread) const;
   std::string unicodeToUtf8(uint32_t unicode) const;
   int utf16ToUnicodeCodepoint(uint16_t utf16, uint32_t *codepoint) const;
+  std::string makePrintable(std::string const &in) const;
 };
 
 // ONLY FOR DUMMYBACKUP
@@ -733,14 +779,14 @@ inline void SignalBackup::addEndFrame()
   d_endframe.reset(new EndFrame(nullptr, 1ull));
 }
 
-inline void SignalBackup::runQuery(std::string const &q, bool pretty) const
+inline void SignalBackup::runQuery(std::string const &q, std::string const &mode) const
 {
   Logger::message(" * Executing query: ", q);
   SqliteDB::QueryResults res;
   if (!d_database.exec(q, &res))
     return;
 
-  std::string q_comm = q.substr(0, STRLEN("DELETE")); // delete, insert and update are same length...
+  std::string q_comm(q, 0, STRLEN("DELETE")); // delete, insert and update are same length...
   std::for_each(q_comm.begin(), q_comm.end(), [] (char &ch) { ch = std::toupper(ch); });
 
   if (q_comm == "DELETE" || q_comm == "INSERT" || q_comm == "UPDATE")
@@ -750,8 +796,12 @@ inline void SignalBackup::runQuery(std::string const &q, bool pretty) const
       return;
   }
 
-  if (pretty)
+  if (mode == "pretty")
     res.prettyPrint(d_truncate);
+  else if (mode == "line")
+    res.printLineMode();
+  else if (mode == "single")
+    res.printSingleLine();
   else
     res.print();
 }
@@ -826,20 +876,21 @@ inline bool SignalBackup::HTMLwriteIndex(std::vector<long long int> const &threa
                                          std::map<long long int, RecipientInfo> *recipient_info, long long int note_to_self_tid,
                                          bool calllog, bool searchpage, bool stickerpacks, bool blocked, bool fullcontacts, bool settings,  bool overwrite,
                                          bool append, bool light, bool themeswitching, std::string const &exportdetails,
-                                         std::vector<std::pair<std::string, std::string>> const &chatfolders) const
+                                         std::vector<std::tuple<long long int, std::string, std::string>> const &chatfolders,
+                                         bool compact) const
 {
   return HTMLwriteIndexImpl(threads, maxtimestamp, directory, "index", recipient_info, note_to_self_tid, calllog, searchpage, stickerpacks, blocked,
-                            fullcontacts, settings, overwrite, append, light, themeswitching, exportdetails, -1, chatfolders);
+                            fullcontacts, settings, overwrite, append, light, themeswitching, exportdetails, -1, chatfolders, compact);
 }
 
 inline bool SignalBackup::HTMLwriteChatFolder(std::vector<long long int> const &threads, long long int maxtimestamp, std::string const &directory,
                                               std::string const &basename,  std::map<long long int, RecipientInfo> *recipient_info, long long int note_to_self_tid,
                                               bool calllog, bool searchpage, bool stickerpacks, bool blocked, bool fullcontacts, bool settings,  bool overwrite,
-                                              bool append, bool light, bool themeswitching, std::string const &exportdetails, long long int chatfolderidx) const
+                                              bool append, bool light, bool themeswitching, std::string const &exportdetails, long long int chatfolderidx,
+                                              std::vector<std::tuple<long long int, std::string, std::string>> const &chatfolders, bool compact) const
 {
   return HTMLwriteIndexImpl(threads, maxtimestamp, directory, basename, recipient_info, note_to_self_tid, calllog, searchpage, stickerpacks, blocked,
-                            fullcontacts, settings, overwrite, append, light, themeswitching, exportdetails, chatfolderidx,
-                            std::vector<std::pair<std::string, std::string>>());
+                            fullcontacts, settings, overwrite, append, light, themeswitching, exportdetails, chatfolderidx, chatfolders, compact);
 }
 
 inline int SignalBackup::utf16CharSize(std::string const &body, int idx) const
@@ -939,6 +990,19 @@ inline std::string SignalBackup::utf8BytesToHexString(std::string const &data) c
   return utf8BytesToHexString(reinterpret_cast<unsigned char const *>(data.data()), data.size());
 }
 
+inline SignalBackup::RecipientInfo const &SignalBackup::getRecipientInfoFromMap(std::map<long long int, RecipientInfo> *recipient_info,
+                                                                                long long int rid) const
+{
+  if (auto found = recipient_info->find(rid); found != recipient_info->end()) [[likely]]
+    return found->second;
+
+  // if (bepaald::contains(recipient_info, rid))
+  //   return (*recipient_info)[rid];
+
+  setRecipientInfo({rid}, recipient_info);
+  return (*recipient_info)[rid];
+}
+
 inline void SignalBackup::TXTaddReactions(SqliteDB::QueryResults const *const reaction_results, std::ofstream *out) const
 {
   if (reaction_results->rows() == 0) [[likely]]
